Package: chipenrich
Type: Package
Title: Gene Set Enrichment For ChIP-seq Peak Data
Version: 2.35.0
Date: 2025-07-22
Authors@R: c(
	person(c("Ryan","P."),"Welch",role = c("aut","cph"),email = "welchr@umich.edu"),
	person("Chee","Lee",role = c("aut"),email = "cheelee@umich.edu"),
	person(c("Raymond","G."),"Cavalcante",role = c("aut"),email = "rcavalca@umich.edu"),
	person("Kai","Wang",role = c("cre"),email = "wangdaha@umich.edu"),
	person("Chris","Lee",role = c("aut"),email = "leetaiyi@umich.edu"),
	person(c("Laura","J."),"Scott",role = c("ths"), email = "ljst@umich.edu"),
	person(c("Maureen","A."),"Sartor",role = c("ths"),email = "sartorma@med.umich.edu"))
Description: ChIP-Enrich and Poly-Enrich perform gene set enrichment testing
	using peaks called from a ChIP-seq experiment. The method empirically
	corrects for confounding factors such as the length of genes,
	and the mappability of the sequence surrounding genes.
biocViews: ImmunoOncology, ChIPSeq, Epigenetics, FunctionalGenomics,
        GeneSetEnrichment, HistoneModification, Regression
License: GPL-3
Imports: AnnotationDbi, BiocGenerics, chipenrich.data, Seqinfo,
        GenomicRanges, grDevices, grid, IRanges, lattice, latticeExtra,
        MASS, methods, mgcv, org.Dm.eg.db, org.Dr.eg.db, org.Hs.eg.db,
        org.Mm.eg.db, org.Rn.eg.db, parallel, plyr, rms, rtracklayer,
        S4Vectors (>= 0.23.10), stats, stringr, utils
Suggests: BiocStyle, devtools, knitr, rmarkdown, roxygen2, testthat
Depends: R (>= 3.4.0)
LazyLoad: yes
Maintainer: Kai Wang <wangdaha@umich.edu>
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Collate: 'assign_peaks.R' 'test_broadenrich.R' 'peaks_per_gene.R'
        'read.R' 'randomize.R' 'setup.R' 'supported.R' 'utils.R'
        'constants.R' 'plot_gene_coverage.R' 'broadenrich.R'
        'test_chipapprox.R' 'test_chipenrich.R'
        'test_chipenrich_slow.R' 'test_fisher.R' 'test_binomial.R'
        'test_approx.R' 'plot_chipenrich_spline.R' 'plot_dist_to_tss.R'
        'chipenrich.R' 'chipenrich_package_doc.R' 'test_polyapprox.R'
        'test_polyenrich_weighted.R' 'test_polyenrich_slow.R'
        'test_polyenrich.R' 'plot_polyenrich_spline.R' 'polyenrich.R'
        'hybrid.R' 'peak_weights.R' 'peaks2genes.R' 'test_proxReg.R'
        'proxReg.R'
git_url: https://git.bioconductor.org/packages/chipenrich
git_branch: devel
git_last_commit: 93dd65a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-06
NeedsCompilation: no
Packaged: 2025-11-06 21:46:26 UTC; biocbuild
Author: Ryan P. Welch [aut, cph],
  Chee Lee [aut],
  Raymond G. Cavalcante [aut],
  Kai Wang [cre],
  Chris Lee [aut],
  Laura J. Scott [ths],
  Maureen A. Sartor [ths]
