# Generated by roxygen2: do not edit by hand

export(assign_peak_segments)
export(assign_peaks)
export(broadenrich)
export(calc_peak_gene_overlap)
export(chipenrich)
export(hybridenrich)
export(load_peaks)
export(num_peaks_per_gene)
export(peaks2genes)
export(plot_chipenrich_spline)
export(plot_dist_to_tss)
export(plot_gene_coverage)
export(plot_polyenrich_spline)
export(polyenrich)
export(proxReg)
export(read_bed)
export(supported_genesets)
export(supported_genomes)
export(supported_locusdefs)
export(supported_methods)
export(supported_read_lengths)
import(GenomicRanges)
import(chipenrich.data)
import(grid)
import(lattice)
import(latticeExtra)
import(methods)
import(mgcv)
import(org.Dm.eg.db)
import(org.Dr.eg.db)
import(org.Hs.eg.db)
import(org.Mm.eg.db)
import(org.Rn.eg.db)
import(parallel)
import(rms)
import(rtracklayer)
importFrom(AnnotationDbi,mappedkeys)
importFrom(BiocGenerics,unlist)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,splitAsList)
importFrom(S4Vectors,subjectHits)
importFrom(Seqinfo,seqinfo)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(plyr,rbind.fill)
importFrom(stats,as.formula)
importFrom(stats,binom.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,fisher.test)
importFrom(stats,fitted)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,qpois)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stringr,str_match)
importFrom(stringr,str_sub)
importFrom(utils,data)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
