# ChIP-Enrich

R package for gene set enrichment testing using ChIP-seq data.

# Description

ChIP-Enrich performs gene set enrichment testing using peaks called from a ChIP-seq experiment. The method empirically corrects for confounding factors such as the length of genes, and the mappability of the sequence surrounding genes.
